import wx
import matplotlib
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
matplotlib.interactive(False)
from matplotlib.pyplot import gcf, setp

#This is the window (frame in WX jargon) used to plot everything 
class PipelineInteractiveFrame(wx.Frame):
  
  def __init__(self, inputs, outputs, dataPlotManager):
    wx.Frame.__init__(self, parent=None, title=dataPlotManager.setWindowTittle(), 
                      size=(1324, 900))
  
    #Initialize the inputs and outputs
    self.inputs          = inputs
    self.outputs         = outputs
    self.dataPlotManager = dataPlotManager
  
    #Get all the frames
    in_sof_splitted = self.inputs.in_sof.split(',')
    self.sof = list()
    for frame in in_sof_splitted:
      self.sof.append(frame[0:frame.rfind(':')])
      
    #Set the parameters to be plotted
    self.paramList, self.paramDesc = dataPlotManager.setInteractiveParameters()
      
    #Split the in_sop port
    self.input_sop = inputs.in_sop.split(',')
  
    #The plotting window
    self.pipePlotsWindow = PipelinePlotsWindow(self, self.sof, dataPlotManager)
  
    #The control area
    self.controlArea = wx.BoxSizer(wx.HORIZONTAL)
  
    #The area for the buttons
    self.buttonArea = wx.BoxSizer(wx.VERTICAL)
  
    #The area for the parameters
    self.parameterArea = wx.StaticBoxSizer(wx.StaticBox(self, -1,'Parameters'),
                                             wx.VERTICAL)
    self.parameterGrid = wx.GridBagSizer(4,4)
  
    #The widgets for the parameters
    self.paramNames   = list()
    self.paramValues  = list()
    self.paramWidgets = list()
    iParam = 0
    for param, desc in zip(self.paramList, self.paramDesc):
      for param_in_sop in self.input_sop:
        if param_in_sop.partition('=')[0] == param:
          self.paramNames.append(param_in_sop.partition('=')[0] )
          self.paramValues.append(param_in_sop.partition('=')[2]) 
          paramName = param_in_sop.partition('=')[0].partition(':')[2]
          paramWidget = ParamWidget(self, label=paramName, \
                                   paramValue=self.paramValues[iParam],
                                   description=desc)
          self.paramWidgets.append(paramWidget)
          iParam = iParam + 1
            
    # The buttons
    self.contButton = wx.Button(self, label="Continue wkf")
    self.contButton.SetToolTip(wx.ToolTip("Accept the current products and pass them to next recipe"))
    self.Bind(wx.EVT_BUTTON, self.OnCont, self.contButton)
    self.repeatButton = wx.Button(self, label="Re-run recipe")
    self.repeatButton.SetToolTip(wx.ToolTip("Re execute the recipe with the new parameters"))
    self.Bind(wx.EVT_BUTTON, self.OnRepeat, self.repeatButton)
    self.exportpdfButton = wx.Button(self, label="Print plot")
    self.exportpdfButton.SetToolTip(wx.ToolTip("Print the plotting canvas"))
    self.Bind(wx.EVT_BUTTON, self.OnPrint, self.exportpdfButton)
    self.helpButton = wx.Button(self, label="Help")
    self.Bind(wx.EVT_BUTTON, self.OnHelp, self.helpButton)
  
    #Pack parameter widgets
    iParam = 0
    for paramWidget in self.paramWidgets:
      self.parameterGrid.Add(paramWidget.paramLabel, (iParam, 0),
          flag=wx.TOP | wx.LEFT | wx.BOTTOM, border=5)
      self.parameterGrid.Add(paramWidget.paramEditText, (iParam, 1),
          flag=wx.EXPAND | wx.LEFT | wx.RIGHT, border=5)
      iParam = iParam + 1
  
    #Pack buttons
    self.buttonArea.Add(self.contButton, 0, \
        wx.ALIGN_CENTER | wx.ALL, border=5)
    self.buttonArea.Add(self.repeatButton, 0, \
        wx.ALIGN_CENTER | wx.ALL, border=5)
    self.buttonArea.Add(self.exportpdfButton, 0, \
        wx.ALIGN_CENTER | wx.ALL, border=5)
    self.buttonArea.Add(self.helpButton, 0, \
        wx.ALIGN_CENTER | wx.ALL, border=5)
  
    #Pack the parameter grid
    self.parameterArea.Add(self.parameterGrid, 0, \
        flag=wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, border=5) 
  
    #Pack the parameters area
    self.controlArea.Add(self.parameterArea, 0, \
        wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT | wx.ALL, border=5)
  
    #Pack the button area
    self.controlArea.Add(self.buttonArea, 0, \
        wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_RIGHT | wx.ALL, border=5)
  
    #The Sizer which packs everything
    sizer = wx.BoxSizer(wx.HORIZONTAL)
    sizer.Add(self.pipePlotsWindow, 1, wx.EXPAND)
    sizer.Add(self.controlArea, 0, wx.EXPAND | wx.ALL, border=5)
      
    self.SetSizer(sizer)
  
  def OnCont(self, event):
    self.outputs.iteration_complete = 'true'
    self.outputs.out_sof            = self.inputs.in_sof
    self.outputs.out_sop            = self.inputs.in_sop
  
    self.Close(True)

  def OnRepeat(self, event):
    #This will update the output parameters based on the entry boxes.
    self.outputs.out_sop_loop = ''
    for param_in_sop in self.input_sop:
      finalValue = param_in_sop
      for i in range(len(self.paramWidgets)):
        if param_in_sop.partition('=')[0] == self.paramNames[i]:
            finalValue = self.paramNames[i] + '=' +  \
                         self.paramWidgets[i].paramEditText.GetValue()
      if self.outputs.out_sop_loop != '':
        self.outputs.out_sop_loop += ','
      self.outputs.out_sop_loop += finalValue
  
    self.outputs.iteration_complete = 'false'
    self.outputs.out_sof_loop       = self.inputs.in_sof_rec_orig
    self.Close(True)

  def OnPrint(self, event):
    self.pipePlotsWindow.canvas.Printer_Print(event=event)
    return

  def OnHelp(self, event):
    window_help_msg = self.dataPlotManager.setWindowHelp()
    general_help_msg = """
      
The window has several parts:\n
   1. Plot area. This area shows how good the reduction was performed
   2. Parameter area. This area shows the parameters used for the execution of the recipe. The parameters can be changed if the recipe has to be re-run with new parameters
   3. Button area. These buttons control the interactivity of the window\n
        a) Continue wkf. This button sends the current results to the next recipe in the workflow.\n
        b) Re-run recipe. This button will execute the predict recipe again with the new parameters\n
        c) Print plot. This button will print the plot area\n
        d) Help. This button shows this help
   4. Top toolbar area. These buttons allow certain interactivity with the plots (zoom, shift, layout) as well as exporting to png 
        """
    dlg = wx.MessageDialog(self, window_help_msg + general_help_msg, 
                           self.dataPlotManager.setWindowTittle()+" Help",
                           wx.OK | wx.ICON_INFORMATION)
    dlg.ShowModal()
    dlg.Destroy()

#This class is responsible for the plotting window (which uses matplotlib)
class PipelinePlotsWindow(wx.Window):
  def __init__(self, frame, sof, dataPlotManager):
    wx.Window.__init__(self, frame)
    self.dataPlotManager = dataPlotManager
      
    #Read the data
    self.dataPlotManager.readFitsData(sof)
      
    #Get the matplotlib canvas
    self.figure = Figure()
    self.canvas = FigureCanvasWxAgg(self, -1, self.figure)
    self.subplotsCreated = False
    self.draw()
    self.canvas.SetToolTip(wx.ToolTip(tip=''))
    self.toolbar = NavigationToolbar2Wx(self.canvas)
    frame.SetToolBar(self.toolbar)
    self.canvas.mpl_connect('axes_enter_event',self.onEnterAxes)
    self.canvas.mpl_connect('axes_leave_event',self.onLeaveAxes)
  
    #React to resizing events 
    self.Bind(wx.EVT_SIZE, self.sizeHandler)
  
  def onEnterAxes(self, event):
    self.canvas.GetToolTip().SetTip(event.inaxes.tooltip)
    self.canvas.GetToolTip().Enable(True)

  def onLeaveAxes(self, event):
    self.canvas.GetToolTip().Enable(False)
  
  def sizeHandler(self, *args, **kwargs):
    self.canvas.SetSize(self.GetSize())
  
  def draw(self):
    if self.subplotsCreated == False:
      self.dataPlotManager.addSubplots(self.figure)
      self.subplotsCreated = True
        
    self.dataPlotManager.plotProductsGraphics(self.figure, self.canvas)
    self.figure.subplots_adjust(wspace=0.40, hspace=0.55, top=0.95, bottom=0.05, left=0.1, right=0.95)

  def repaint(self):
    self.canvas.draw()
  

#This class is used to display the label and edit box of a parameter
class ParamWidget:
  def __init__(self, parent, label, paramValue, description):
    self.paramLabel = wx.StaticText(parent, label=label)
    self.paramEditText = wx.TextCtrl(parent, -1, style=wx.TE_PROCESS_ENTER)
    self.paramEditText.SetValue(paramValue)
    self.paramEditText.SetToolTipString(description)
  
    #This binding is unnecessary, since the OnRepeat directly access the
    #GetValue() of paramEditText. I keep it here in case that
    #the desired behaviour is that you have to press Enter to actually edit the
    #value
    self.paramEditText.Bind(wx.EVT_TEXT_ENTER, self.paramEditTextHandler)
  
    self.paramValue = paramValue
  
  def paramEditTextHandler(self, evt):
    value = self.paramEditText.GetValue()
    self.paramValue = value
  
#This class presents the wx application interface
class PipelineInteractiveApp(wx.App):

  def __init__(self, inputs, outputs, dataPlotManager):
    self.inputs           = inputs
    self.outputs          = outputs
    self.dataPlotManager = dataPlotManager
    wx.App.__init__(self,0)
  
  def OnInit(self):
    'Create the main window and insert the custom frame'
    self.frame = PipelineInteractiveFrame(self.inputs, self.outputs, self.dataPlotManager)
    self.frame.Show(True)
    return True
